#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    PRODUCTS=${product_dir}:${PRODUCTS}
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------
set -x
package=python
origpkgver=v3_7_1
pkgver=${origpkgver}
ssibuildshims_version=v1_04_12
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pythonMmVer=`echo ${pkgdotver} | sed -e 's/^\([0-9]\{1,\}\.[0-9]\{1,\}\).*$/\1/'`
pkgtarname=Python-${pkgdotver}
set  +x
get_this_dir

get_ssibuildshims

source define_basics NO_QUAL NO_DEBUG_SOURCE
scriptsdir=${product_dir}/${package}/${pkgver}/scripts

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# setup this package
setup -B ${package} ${pkgver} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "fake setup failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

if [ -z ${PYTHON_ROOT} ]
then
   echo "ERROR: PYTHON_ROOT is not defined"
   exit 1
else
   echo "PYTHON_ROOT: ${PYTHON_ROOT}"
fi

if [ -z ${SQLITE_INC} ]
then
   echo "ERROR: failed to setup sqlite"
   exit 1
else
   echo "SQLITE_INC: ${SQLITE_INC}"
fi

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tgz

cd ${sourcedir}/${pkgtarname}

declare -a patches=(${patchdir}/${pkgtarname}.patch)

# Note: sysconfigdata.patch is used by fixpython.sh, whose invocation is
# inserted into the procedure by the main patch above. Do *not* attempt
# to apply it separately.
if [ ${OS1} = "Linux" ]; then
  if [ "$(lsb_release -i -s)" = "SUSE LINUX" ]
  then
    patches[${#patches[@]}]=${patchdir}/SUSE.Modules.panel.patch
  fi
fi

for patch in "${patches[@]}"; do
  patch ${DEBUG_PATCHES:+--verbose} -b -p0 < "${patch}" || ssi_die "application of patch ${patch} failed."
done

(( ${DEBUG_PATCHES:-0} )) && exit 1

configure_opts=(--prefix="${pkgdir}" \
                        --enable-ipv6 \
                        --without-ensurepip \
                        --enable-shared \
               )

if [[ "${OS1}" == Darwin ]]; then
  # We need to specify the installed location of SQLite so that the
  # build finds ours, not a system install.
  if [ -d /usr/local/opt/openssl ]; then
    configure_opts+=(--with-openssl=/usr/local/opt/openssl --with-ssl-default-suites=openssl)
  fi

  configure_opts+=(--without-gcc --enable-optimizations)
  export CC=clang
  export CXX=clang++
fi

mkdir -p ${blddir} || ssi_die "Unable to create build directory ${blddir}"
cd "${blddir}" || ssi_die "Unable to change directory to ${blddir}"

"${sourcedir}/${pkgtarname}/configure" "${configure_opts[@]}" || \
  ssi_die "configure failed."

make -j ${ncore} || ssi_die "make failed."
make install || ssi_die "make install failed."

# replace utilities with scripts which use #!/usr/bin/env python
if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

# Set permissions on $2, using $1 as reference.
function set_perms() {
  local ref="${1}"
  local file="${2}"
  if [[ "${OS1}" == Darwin ]]; then
    chmod $(stat -f %p "${ref}") "${file}"
  else
    chmod --reference="${ref}" "${file}"
  fi
}

# Fix the hardwired shebang line on some scripts
function fixshebang() {
  local prog="${1}"
  while [[ -L "${prog}" ]]; do
    prog=$(readlink "${prog}")
  done
  # Check whether it's a script.
  file "${file}" | grep -q -i 'script' || return
  mv "${prog}"{,.bak}
  cat "${prog}.bak" | sed -e '1s%^#!.*python[^ 	]*%#!/usr/bin/env python%' > "${prog}"
  set_perms "${prog}"{.bak,}
  rm -f "${prog}.bak"
}

# Fix a single file in the current directory
function linkandfixshebang() {
  local file=${1}
  local suffix=${2}
  # Generic link (no version suffix).
  [[ "${file}" == *"${pythonMmVer}"${suffix} ]] && \
    ln -s "${file}" "${file%${pythonMmVer}${suffix}}${suffix}" 2>/dev/null
  fixshebang "${file}"
}

# Fix shebang lines.
cd "${pkgdir}/bin/"
for file in *; do
  if [[ "${file}" == python*-config ]]; then
    suffix=-config
  else
    unset suffix
  fi
  linkandfixshebang "${file}" "${suffix}"
done

cd "${pkgdir}/lib/python${pythonMmVer}/config"-* && fixshebang python-config.py

cd "${pkgdir}"

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}  -z ${product_dir}
echo "${package} is installed at ${PYTHONHOME}"
echo ""

# fix paths in ${PYTHONHOME}/lib/python${pythonMmVer}/config*/Makefile
config_file=($(find "${PYTHONHOME}/lib/python${pythonMmVer}/config"* -name Makefile))
(( ${#config_file[@]} == 1 )) || ssi_die "Unable to locate an unique config Makefile."
config_file="${config_file[0]}"

fixscript=`mktemp "\${TMPDIR:-/tmp}/python-fix.XXXXXXXXXX"`
trap "rm -f \"$fixscript\" 2>/dev/null" EXIT

cat > "${fixscript}" <<EOF
#!/bin/bash
cfg="${config_file}"
while [[ -L "\${cfg}" ]]; do
   prog=\$(readlink "\${cfg}")
done

mv "\${cfg}" "\${cfg}.bak"
cat "\${cfg}.bak" \\
      | sed 's%${PYTHON_INCLUDE}%\$(PYTHON_INCLUDE)%g' \\
      | sed 's%${PYTHON_LIB}%\$(PYTHON_LIB)%g' \\
      | sed 's%${PYTHONHOME}%\$(PYTHONHOME)%g' \\
      | sed 's%LD_LIBRARY_PATH=${PYTHON_DIR}/build/${flvr}%LD_LIBRARY_PATH=\$(PYTHON_LIB)%' \\
      | sed 's%${PYTHON_DIR}/src\b%\$(PYTHON_DIR)/src%' \\
      | sed 's%${PYTHON_DIR}%\$(PYTHON_DIR)%g' \\
      | sed 's%${SQLITE_INC}%\$(SQLITE_INC)%g' \\
      | sed 's%${SQLITE_LIB}%\$(SQLITE_LIB)%g' \\
      | sed 's%${SQLITE_FQ_DIR}%\$(SQLITE_FQ_DIR)%g' \\
      > "\${cfg}" && \
rm -f "\${cfg}.bak"

EOF

chmod u+rx ${fixscript} && \
  ${fixscript} || \
  {
  trap "" EXIT
  ssi_die "Unsuccessful execution of fixup script ${fixscript}"
}

# Normal cleanup
rm -f "${fixscript}" 

# Test that the library has built all the necessary components
echo "The Python we are running is at:" $(type python)
echo "Preparing to test loading of Python Standard Library modules"
python ${scriptsdir}/test-standard-modules.py || \
 ssi_die "unable to find all expected Python Standard Library modules."
echo "Succeded in test loading of Python Standard Library modules"

# Now install pip, and use that to install setuptools. We will get
# whatever version of pip is 'current'. get-pip.py is a bootstrapping
# tool, and will download whatever version is current. In order to
# control the version of pip we have, after bootstrapping with
# get-pip.py, we immediately downgrade to the version we specify.
cd "${blddir}" || ssi_die "Unable to change directory to ${blddir}"

curl "https://bootstrap.pypa.io/get-pip.py" -o "get-pip.py" || ssi_die "Unable to download git-pip.py"
python get-pip.py --no-cache-dir --no-wheel --no-setuptools --prefix ${pkgdir} || ssi_die "Failed executing 'python get-pip.py'"
pip install -I pip==18.1 || ssi_die "Failed to install required pip version"
pip install setuptools==40.6.2 || ssi_die "Failed to install setuptools"
pip install --no-cache-dir -r ${scriptsdir}/requirements.txt --prefix ${pkgdir} || ssi_die "Failed executing 'pip install ..."
rm get-pip.py || ssi_die "Failed to remove 'get-pip.py'"

cd "${pkgdir}/bin"
for file in *; do fixshebang "${file}"; done

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver}  NO_QUAL
fi

exit 0
